%% Diffusion Coda Approximation
clear all; close all; warning off all

% Admin

% Event files
FileList = dir('ascii-files\*.txt');
FileList = struct2cell(FileList)';
FileList = sortrows(FileList,1);
TOT = size(FileList,1);
load inputs.mat

% Windowing

cd ascii-files
file = dlmread(char(FileList(1,1)),'%f');
cd ..
mint = min(file(:,1));
maxt = max(file(:,1));
trange = maxt - mint;
unitspert = size(file,1) / trange;
numwin = 5; % Define number of windows

soilnoise = 1.4; % Noise threshhold

sn = 500; % Noise window size

fs = [128 128 4 8 8 8]; % Smoothing

srate = 1000; % Sampling rate
freq = [3:3:18]; % Frequencies
tmax = [65 60 45 40 35 30]; % Total analysis length per frequency (tmax)

% Number of stations
nray = size(unique(station),2);

% Energy windows

nfreq = size(freq,2);   % Number of Frequencies
W = zeros(TOT,nfreq,numwin); % Left side of diffusion equation
noise = zeros(TOT,numwin,nfreq); % Signal to noise ratios

%% Left side of diffusion equation

for i = 1:TOT
    tic;
    
    cd ascii-files
    filename = [num2str(i),'.txt'] % Event
    signal = dlmread(filename,'%f'); % Signals
    signal = signal(:,2);
    signal = signal - mean(signal); % Normalise around zero
    t1 = round((tempi(i,1) - mint)*unitspert,0); % P-arrival
    t2 = round((tempi(i,2) - mint)*unitspert,0); % P-max
    %t2 = t2+(t2-t1);
    if t1 < 0
        t1 = 1000;
    end

    cd ..
   
            % Frequency filtering
            for n = 1:nfreq
                
                cf = freq(n); % Filter at frequency
                Wn = ([cf-cf/10 cf+cf/10])/srate; %frequency band
                [z,p,k] = butter(6,Wn,'bandpass'); %butter filter
                [sos,g] = zp2sos(z,p,k);      % Convert to SOS form
                Hd = dfilt.df2tsos(sos,g);   % Create a dfilt object 
                
                cursorn1 = t1 - sn; % Starting sample for noise
                cursorn2 = t1; % End of noise window
                if cursorn1 < 0
                    cursorn1 = 1;
                end

                signaln = signal(cursorn1:cursorn2); % Signal crop
                fsignaln = filter(Hd,signaln); % Filter noise
                efsignaln = abs(hilbert(sqrt(fsignaln.^2))); % Measure noise energy
                
                signalnoise = efsignaln; % Noise energy values
                                              
                % Measuring trace energy in windows from P-max
                
                analysislength = tmax(n); % Total window length in time units
                analysislength2 = (unitspert * analysislength);% - t2; % Total window length in step units
                kmax = numwin; % Number of windows
                delay = round(analysislength2 / kmax,0); % Window size
                
                window = 0; % Windowing reset
                
                for k = 1:kmax
                    
                    cursorc1 = t1 + window;% Start window for coda sample
                    cursorc2 = cursorc1 + delay; % End of coda window
                    

                
                    if cursorc1 > size(signal,1) % Window exceeds trace
                        W(i,n,k) = NaN; % Energy value for window
                        noise(i,k,n) = NaN;
                        winmid(i,k,n) = NaN;
                        continue
                    end
                                                        
                    if cursorc2 > size(signal,1) % If final window exceeds trace
                        cursorc2 = size(signal,1); % Set to max trace length
                    end
                    
                    winmid(i,k,n) = (mint + ((cursorc2 - (delay/2)) / unitspert)) - tempi(i,2);%cell2mat(pktimes(j,2,i)); % Centre of window from P-arrival
                    signalc = signal(cursorc1:cursorc2); % Signal crop
                    fsignalc = filter(Hd,signalc); % Filter coda
                    efsignalc = abs(hilbert(fsignalc).^2); % Hilbert
                    sefsignalc = smooth(efsignalc,fs(n))/cf*srate; % Smoothing
                    
                    % Diffusion approximation
                    
                    dLeft = log(sefsignalc.*winmid(i,k,n).^1.5); % Left side of diffusion equation
                    W(i,n,k) = mean(dLeft); % Energy value for window
                    noise(i,k,n) = mean(sefsignalc) / mean(signalnoise); % Signal to noise ratio
                    
                    window = window + delay; % Next window
                    
                    if t1 > 2000 % Picking error
                        W(i,n,k) = NaN; 
                        noise(i,k,n) = NaN;
                        winmid(i,k,n) = NaN;
                    elseif t1 < 750
                        W(i,n,k) = NaN; 
                        noise(i,k,n) = NaN;
                        winmid(i,k,n) = NaN;
                    end
                    
                end
                
            end
            
    % Clock
    
    time(i) = toc;
    ttime = mean(time)*TOT;
    if i < 31
    timeleft = round(ttime - (mean(time)*i),0);
    else
        timeleft = round(ttime - (mean(time(i-30:i))*i),0);
    end
    
    display([num2str(timeleft),' seconds left for energy values'])
    
end

save diffusionenergy.mat W noise winmid

%% Inversion

% Admin

load diffusionenergy.mat
load inputs.mat
kmax = size(W,3);
soilnoise = 1.4;
station = station';

A = ones(kmax,3); % Model vector
bdE = zeros(size(W,1),3,size(W,2)); % Inversion matrix

% Right side of diffusion equation

for i = 1:size(W,1)
          
    stn = char(station(i));     

    if stn == 'JRO' % Simplified distances
        dst=10;
    elseif stn == 'FL2'
        dst=10;
    elseif stn == 'ELK'
        dst=10;
    elseif stn == 'MTM'
        dst=10;
    elseif stn == 'CDF'
        dst=10;
    elseif stn == 'TDL'
        dst=10;
    elseif stn == 'SOS'
        dst=4;
    elseif stn == 'EDM'
        dst=4;
    elseif stn == 'JUN'
        dst=4;
    elseif stn == 'SHW'
        dst=4;
    elseif stn == 'YEL'
        dst=4;
    elseif stn == 'HSR'
        dst=4;
    else
        dst=4;
    end
%     
%     if stn == 'JRO' % Estimated Distances
%         dst=10;
%     elseif stn == 'FL2'
%         dst=10.75;
%     elseif stn == 'ELK'
%         dst=12.45;
%     elseif stn == 'MTM'
%         dst=11.6;
%     elseif stn == 'CDF'
%         dst=12.6;
%     elseif stn == 'TDL'
%         dst=11.8;
%     elseif stn == 'SOS'
%         dst=4.5;
%     elseif stn == 'EDM'
%         dst=5;
%     elseif stn == 'JUN'
%         dst=5;
%     elseif stn == 'SHW'
%         dst=5.1;
%     elseif stn == 'YEL'
%         dst=4.1;
%     elseif stn == 'HSR'
%         dst=4.75;
%     elseif stn == 'STD'
%         dst=5.2;
%     else
%         dst=4;
%     end
        
        for n = 1:size(W,2)
            
                A(:,2) = winmid(i,:,n);    % t1
                A(:,3) = 1./winmid(i,:,n); % t2                 
        
            pn = find(noise(i,:,n) >= soilnoise); % Select only windows with minimum noise
            if isempty(pn)
                bdE(i,1:3,n) = NaN;
                continue
            end
            
            clear d_single m_single
            
            for d = 1:size(pn,2)
                d_single(d) = W(i,n,pn(d)); % Data vector
            end
            
            m_single = lsqlin(A(pn,:),d_single); % Model vector
            
            bdE(i,1,n) = real(-m_single(2)); % Attenuation
            bdE(i,2,n) = real(-(dst^2/(4*m_single(3)))); % Diffusivity
            bdE(i,3,n) = real((4*pi*(bdE(i,2,n)))^1.5)*exp(m_single(1)); % Source energy
            
            clear pn
        end
        
    end
       
save diffusion.mat bdE  % row = event, column = bdE, n = frequency

%% Plots

clear all; close all;

load diffusion.mat
%load diffusionenergy.mat

load inputs.mat
freq = [3:3:18];
nBins = 10;
station = station';

% Diffusion

for j = 1:size(bdE,3)

atten = bdE(:,1,j);
diff = bdE(:,2,j);
E0 = bdE(:,3,j);

stn = unique(station);

for i = 1:size(stn,1)
    
    a = strfind(station,stn(i)); %  Finds traces for current station
    indices = find(~cellfun(@isempty,a));
    diff1 = diff(indices);
    atten1 = atten(indices);
    E01 = E0(indices);
    
    b = isnan(diff1); % Removes NaN
    b2 = isnan(atten1);
    b3 = isnan(E01);
    indices = find(b == 0);
    indices2 = find(b2 ==0);
    indices3 = find(b3 ==0);
    diff1 = diff1(indices);
    atten1 = atten1(indices2);
    E01 = E01(indices3);
    
    b = isinf(E01); % Removes infinity values
    indices = find(b == 0);
    E01 = E01(indices);
    b = isinf(diff1);
    indices = find(b == 0);
    diff1 = diff1(indices);
    
    display('diff')
    nBins = round(size(diff1,1)/2,0)-1; % At least 2 traces per bin
    allnumtracediff(i,j) = size(diff1,1);
    [N,E] = histcounts(diff1,nBins); % Hist count
    %figure; hist(diff1,nBins);set(gcf,'color','white');title(['Diffusion for ',char(stn(i)),' at ',num2str(freq(j)),'Hz'])
         ylabel('Number of events')
     xlabel('Value')
    indices = find(N == max(N));    % Define bin limits
    if size(indices,2) > 1
        y(1,1) = E(indices(1)); % Define averaging limits (error)
        y(1,2) = E(indices(end));
    else
        y(1,1) = E(indices);
        y(1,2) = E(indices + 1);
    end
    indices = find(diff1 >= y(1,1) & diff1 <= y(1,2)); % Find values within limits
    diff1 = diff1(indices);
    numtracediff(i,j) = size(diff1,1);
    diffstd(i,j) = std(diff1);
    differr(i,j,1) = y(1,1); % Error bars
    differr(i,j,2) = y(1,2);
    clear y
    
    display('atten')
    nBins = round(size(atten1,1)/2,0)-1;
    allnumtraceatten(i,j) = size(atten1,1);
    [N,E] = histcounts(atten1,nBins);
    % figure; hist(atten1,nBins);set(gcf,'color','white');title(['Attenuation for ',char(stn(i)),' at ',num2str(freq(j)),'Hz'])
     ylabel('Number of events')
     xlabel('Value')
    indices = find(N == max(N));
    if size(indices,2) > 1
        y(1,1) = E(indices(1));
        y(1,2) = E(indices(end));
    else
        y(1,1) = E(indices);
        y(1,2) = E(indices + 1);
    end
    indices = find(atten1 >= y(1,1) & atten1 <= y(1,2));
    atten1 = atten1(indices);
    numtraceatten(i,j) = size(atten1,1);
    attenstd(i,j) = std(atten1);
    atterr(i,j,1) = y(1,1);
    atterr(i,j,2) = y(1,2);
    clear y
    
    display('e0')

    nBins = round(size(E01,1)/2,0)-1;
    allnumtracee0(i,j) = size(E01,1);
    [N,E] = histcounts(real(E01),nBins);
    % figure; hist(E01,nBins);set(gcf,'color','white');title(['Source Energy for ',char(stn(i)),' at ',num2str(freq(j)),'Hz'])
    indices = find(N == max(N));
    if size(indices,2) > 1
        y(1,1) = E(indices(1));
        y(1,2) = E(indices(end));
    else
        y(1,1) = E(indices);
        y(1,2) = E(indices + 1);
    end
    indices = find(E01 >= y(1,1) & E01 <= y(1,2));
    E01 = E01(indices);
    numtracee0(i,j) = size(E01,1);
    e0std(i,j) = std(E01);
    e0err(i,j,1) = y(1,1);
    e0err(i,j,2) = y(1,2);
    clear y

    diffusion(i,j) = mean(diff1); % Row = station, column = frequency
    attenuation(i,j) = mean(atten1);
    sourceenergy(i,j) = mean(E01);
    
end
end

diff = zeros(1,6);
figure % Diffusion Plot
hold on
for i = [2 5 6 8 9 10 12]
    errorbar(freq,diffusion(i,:),diffstd(i,:),'k')
    set(gcf,'color','white')
    xlabel('Frequency (Hz)')
    ylabel('Diffusion (km^{2}/s)')
 %plot(freq,diffusion(i,:))
 title('Diffusion on volcano')
 ylim([-0.6 2.5])
 
 diff = diff + diffusion(i,:);
 
end
ondiff = diff./7;
onmfp = (ondiff./(2*3)) .* 1000
onmfpmean = (mean(ondiff)/(2*3)) * 1000
ontfp = ((3.*ondiff)./2) .*1000


figure % Attenuation Plot
hold on
for i = [2 5 6 8 9 10 12]
    
 errorbar(freq,attenuation(i,:),attenstd(i,:),'k')
 set(gcf,'color','white')
 xlabel('Frequency (Hz)')
 ylabel('Attenuation (1/s)')
 title('Attenuation on volcano')
 ylim([-0.25 0.25])

end

figure % Source Energy Plot
hold on
for i = [2 5 6 8 9 10 12]
 errorbar(freq,sourceenergy(i,:),e0std(i,:),'k')
 set(gcf,'color','white')
 xlabel('Frequency (Hz)')
 title('Source Energy on volcano')
end

diff = zeros(1,6);
figure
hold on
for i = [1 3 4 7 11]
    errorbar(freq,diffusion(i,:),diffstd(i,:),'k')
    set(gcf,'color','white')
    xlabel('Frequency (Hz)')
    ylabel('Diffusion (km^{2}/s)')
 %plot(freq,diffusion(i,:))
 title('Diffusion off volcano')
 ylim([-0.6 2.5])
 
  diff = diff + diffusion(i,:);
end

offdiff = diff./5;
offmfp = (offdiff./(5*3)) .* 1000
offmfpmean = (mean(offdiff)/(5*3)) * 1000
offtfp = ((3.*offdiff)./5).* 1000

figure
hold on
for i = [1 3 4 7 11]
    
 errorbar(freq,attenuation(i,:),attenstd(i,:),'k')
 set(gcf,'color','white')
 xlabel('Frequency (Hz)')
 ylabel('Attenuation (1/s)')
 title('Attenuation off volcano')
 ylim([-0.25 0.25])

end

figure
hold on
for i = [1 3 4 7 11]
 errorbar(freq,sourceenergy(i,:),e0std(i,:),'k')
 set(gcf,'color','white')
 xlabel('Frequency (Hz)')
 title('Source Energy off volcano')
end
save meanbdE.mat diffusion  attenuation  sourceenergy

for i = 1:size(attenstd,1)
    for j = 1:size(attenstd,2)
        attne(i,j) = cellstr([num2str(round(diffusion(i,j),3)),'  ',num2str(round(diffstd(i,j),3)),' (',num2str(numtracediff(i,j)),')']);
    end
end




